from Globals import DTMLFile, MessageDialog, Persistent, InitializeClass
import OFS
import Acquisition
import AccessControl
from Products import ZCatalog
import base
from StructuredText.StructuredText import HTML
from cgi import escape
import string
import utilities
# The extra entry in sys.path is needed for
# the docutils package
import sys
sys.path.append(SOFTWARE_HOME + '/Products/IssueDealer')
from docutils.core import publish_string
import session_manager
from DateTime import DateTime

def manage_add_issue(self, id=None, title='', contents='',
                     state='', type='', referrer='', REQUEST=None):
    """Add an Issue."""
    if id is None:
        id = self.get_unique_id()
    issue_ = issue(id, title, contents, state, type,
                   creator=self.get_user().get_id(),
                   owner=self.get_user().get_id())
    self._setObject(id, issue_)
    issue_ = self._getOb(id)
    try:
        self.index_object()
    except AttributeError:
        pass
    if referrer and REQUEST is not None:
        REQUEST.RESPONSE.redirect(referrer)
    else:
        if REQUEST is not None:
            return self.manage_main(self, REQUEST)
        else:
            return issue_

# Has to be imported here, because of 'cross importing'
# between the issue and relation modules.
import relation

def manage_add_issue_edit(self, id=None, title='', contents='', referrer='',
                          state='', type='', REQUEST=None):
    """Adds an issue and redirects to the edit page."""
    if id is None:
        id = self.get_unique_id()
    manage_add_issue(self, id=id, title=title, contents=contents,
                     referrer=referrer, state=state, type=type,
                     REQUEST=REQUEST)
    REQUEST.RESPONSE.redirect(self.absolute_url() + '/' + id + '/issue_edit')

class issue(
    ZCatalog.CatalogAwareness.CatalogAware,
    OFS.Folder.Folder,
    Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager,
    base.base,
    session_manager.session_manager,    
    ): 
    """Issue class.

    An issue contains information relating to a task, an
    idea, or whatever.
    """

    meta_type = 'Issue'

    # Backwards compatability
    state = 'open'
    type = 'undefined'
    creator = 'unknown'
    owner_ = 'unknown'
    deleted = 0
    importance = 1
    stx = 1
    created = DateTime("2003/01/01")

    # Needed for cutting and pasting objects; this
    all_meta_types = [
        {'visibility': 'Global',
         'interfaces': [],
         'action': 'manage_add_issue',
         'permission': 'Add Issues',
         'name': 'Issue',
         'product': 'Issue Dealer',
         'instance': ''},
        {'visibility': 'Global',
         'interfaces': [],
         'action': 'manage_add_relation',
         'permission': 'Add Relations',
         'name': 'Relation',
         'product': 'Issue Dealer',
         'instance': ''},
    ]

    manage_options = (OFS.Folder.Folder.manage_options[0],) + \
        (
            {'label': 'View',       'action': ''},
            {'label': 'Security',   'action': 'manage_access'},
        )

    security = AccessControl.ClassSecurityInfo()

    security.declareProtected('Use Issue Dealer',
      'index_html', 'manage_add_issue',
      'manage_add_issue_edit', 'issue_edit')
    index_html = DTMLFile('index', globals())
    manage_add_issue = manage_add_issue
    manage_add_issue_edit = manage_add_issue_edit
    manage_add_relation = relation.manage_add_relation
    issue_edit = DTMLFile('issue_edit', globals())

    def __init__(self, id, title='', contents='', 
                 state='', type='', creator='', owner=''): 
        """initialise a new instance of Issue"""
        self.id = id
        self.title = title
        self.contents = contents
        self.state = state or 'open'
        self.type = type or 'undefined'
        self.creator = creator or 'unknown'
        self.owner_ = owner or 'unknown'
        self.created = DateTime()

    security.declarePrivate('set')
    def set(self, name, value):
        """Alias for attribute setting."""
        exec("self.%s = value" % name)

    security.declarePrivate('render_stx_as_html')
    def render_stx_as_html(self, size=None):
        """Renders the contents as reSTX."""
        if size is not None:
            contents = self.contents[:size]
        else:
            contents = self.contents
        try:
            return publish_string(contents, writer_name='html_inline',
              parser_name='rst', settings_overrides={'input_encoding':'iso-8859-1',
                                                     'output_encoding':'iso-8859-1'})
        except:
            # Failed at returning the text as RST HTML,
            # try regular text instead.
            utilities.log("Failed at converting STX text to HTML")
            utilities.log(contents)
            import sys, traceback, string
            type, val, tb = sys.exc_info()
            utilities.log(string.join(
                traceback.format_exception(type, val, tb), ''))
            return self.render_contents_as_text(size=size)

    security.declarePrivate('render_contents_as_text')
    def render_contents_as_text(self, size=None):
        """Renders the contents as text, keeping whitespace indentation."""
        if size is not None:
            contents = self.contents[:size]
        else:
            contents = self.contents
        contents = contents.replace('\r\n', '\n')
        contents = contents.replace('\n\r', '\n')
        contents = escape(contents)
        lines = contents.split('\n')
        new_lines = []
        for line in lines:
            if not line:
                new_lines.append(line)
                continue
            new_line = ''
            if line[0] == ' ':
                for character in line:
                    if character == ' ':
                        new_line += '&nbsp;'
                    else:
                        new_line += character
            else:
                new_line = line
            new_lines.append(new_line)
        text = string.join(new_lines, '<br />')
        return utilities.hyperlink_urls(text)

    security.declareProtected('Use Issue Dealer', 'manage_edit')
    def manage_edit(self, title, contents, state, type,
                    owner, importance, stx=0, REQUEST=None):
        """Edits the object."""
        self.set('title', title)
        self.set('contents', contents)
        self.set('type', type)
        self.set('stx', stx)
        # State *has* to be changed after type
        self.change_state(state)
        self.set('owner_', owner)
        self.set('importance', importance)
        self.index_object()
        if REQUEST is not None:
            if REQUEST['action'] == 'Save and view':
                REQUEST.RESPONSE.redirect(self.absolute_url())
            else:
                REQUEST.RESPONSE.redirect(self.absolute_url() + '/issue_edit')

    # Untested
    security.declarePrivate('get_possible_issue_types')
    def get_possible_issue_types(self):
        """Returns a sequence of issue types, depending on what
        type of Issue the containing Issue is."""
        try:
            parent_type = self.getParentNode().type
        except AttributeError: # It's the Issue Dealer instance
            parent_type = None
        if parent_type in ('goal', 'problem', 'question'):
            return ('alternative', 'goal', 'idea',
                    'info', 'problem', 'question', 'solution', 'undefined')
        else:
            return ('goal', 'idea', 'info', 'problem', 'question', 'undefined')

    # Untested
    security.declarePrivate('get_relative_importance')
    def get_relative_importance(self):
        """Returns the relative importance of the Issue;  calculated
        by the importance of containing Issues and self."""
        return 1

    # Untested
    security.declarePrivate('get_relative_state')
    def get_relative_state(self):
        """Returns the relative state of the Issue."""
        if self.state == 'open':
            try:
                return self.getParentNode().get_relative_state()
            except AttributeError:
                return self.state
        else:
            return self.state

    # Untested
    security.declarePrivate('change_state')
    def change_state(self, new_state, from_parent=0):
        """Changes the state of the object and, if necessary, the
        state of contained objects and the state of the containing object."""
        current_state = self.state
        if new_state != current_state:
            if self.type in ('goal', 'solution', 'question'):
                if new_state == 'open' and current_state == 'closed':
                    # It has been re-opened and we'll have to undo
                    # the state of contained alternatives and solutions,
                    # if any.
                    issues = self.get_issues()
                    for issue in issues:
                        if issue.type == 'solution':
                            if issue.state == 'closed':
                                issue.change_state('open',
                                  from_parent=1)
                        if issue.type == 'alternative':
                            if issue.state == 'closed':
                                issue.change_state('open',
                                  from_parent=1)
            if self.type == 'solution':
                if new_state == 'closed':
                    # It has resolved/reached the containing
                    # problem or goal.
                    if not from_parent:
                        self.getParentNode().change_state('closed')
                elif new_state == 'open' and current_state == 'closed':
                    # The solution has been re-opened, and
                    # will open the containing goal/problem
                    # as well.
                    if not from_parent:
                        self.getParentNode().change_state('open')
                elif new_state == 'discarded' and current_state == 'closed':
                    # The selected solution has been discarded, and
                    # will open the containing goal/problem
                    if not from_parent:
                        self.getParentNode().change_state('open')
            if self.type == 'alternative':
                if new_state == 'closed':
                    # This Issue has been chosen as the alternative
                    # for the containing goal/problem.
                    #
                    # Won't be called when the containing Issue
                    # is re-opened, so we don't check from_parent
                    # here.
                    self.getParentNode().change_state('closed')
                    issues = self.getParentNode().get_issues()
                    # Discard all other alternatives
                    for issue in issues:
                        if issue.id != self.id:
                            if issue.type == 'alternative' and \
                              issue.state == 'open':
                                issue.change_state('discarded')
                if new_state == 'open':
                    if current_state == 'discarded':
                        # If the Issue was discarded, but is now
                        # an open alternative again, do nothing.
                        pass
                    if current_state == 'closed':
                        # Now we've gone from being the alternative
                        # of choice to being an alternative, and
                        # the containing problem/goal must be
                        # re-opened.
                        if not from_parent:
                            self.getParentNode().change_state('open')
            if self.type == 'problem':
                # If there are open problems related to the Issue,
                # it has to be open
                if new_state == 'open':
                    self.getParentNode().change_state('open')
                elif new_state == 'closed':
                    # Can't close the containing Issue
                    pass
            if self.type == 'goal':
                # If there are open goals related to the Issue,
                # it has to be open
                if new_state == 'open':
                    self.getParentNode().change_state('open')
                elif new_state == 'closed':
                    # Can't close the containing Issue
                    pass
            self.set('state', new_state)
            # Some objects will be indexed twice, as we
            # index ourselves and changed parent objects
            # index contained objects; that's OK.
            self.index_object()
            
    security.declarePrivate('render_html_title_and_link')
    def render_html_title_and_link(self, size=None, show_session=1):
        """Renders a coloured link, depending on the type (and state) of the Issue."""
        color = ''
        if self.state == 'open':
            if self.type == 'problem':
                if self.importance == 2:
                    color = 'red'
                else:
                    color = 'yellow'
            else:
                color = 'green'
        else:
            color = 'green'
        title = self.get_title()
        if self.id in self.get_session().marks and show_session:
            title = "* " + title
        if color:
            html = self.create_hyperlink(self.absolute_url(), title,
              klass=color, size=size, onmouseout="hide('title_%s')" % self.id,
              onmouseover="show('title_%s')" % self.id)
            if size is not None and len(title) > size:
                html += """<div class="hidden" id="title_%s">%s</div>""" %\
                  (self.id, escape(title))
        else:
            html = self.create_hyperlink(self.absolute_url(), title)
        return html

    security.declarePrivate('render_contents')
    def render_contents(self, size=None):
        """Renders the contents."""
        if not self.contents.strip():
            return ""
        if self.stx:
            if self.get_user_preferences().show_entire_issue:
                return self.render_stx_as_html()
            elif (self.REQUEST.has_key('expand_text')
                    and self.REQUEST['expand_text']):
                return self.render_stx_as_html() + \
                       ' <a href="?expand_text:int=0">Less..</a>'
            else:
                contents = self.render_stx_as_html(size=size)
                if size is not None and len(self.contents) > size:
                    contents += ' <a href="%s?expand_text:int=1">More..</a>' % \
                                self.absolute_url()
                return contents
        else:
            return self.render_contents_as_text()

    security.declarePrivate('render_importance_level_widget')
    def render_importance_level_widget(self):
        """Returns a widget for handling importance levels."""
        widget = """<select name="importance:int">"""
        for key, value in self.get_importance_levels():
            if self.importance == key:
                widget += """<option value="%s" selected>%s</option>""" % \
                          (key, value.capitalize())
            else:
                widget += """<option value="%s">%s</option>""" % \
                          (key, value.capitalize())
        return widget + "</select>"

    security.declarePrivate('render_importance')
    def render_importance(self):
        """Renders the importance."""
        for key, value in self.get_importance_levels():
            if key == self.importance:
                return value
        else:
            return 'Unknown'

    security.declarePrivate('render_owner')
    def render_owner(self):
        """Renders the owner."""
        if self.owner_:
            return self.owner_.capitalize()
        else:
            return 'Unknown'

    security.declarePrivate('get_state_importance_title_sort')
    def get_state_importance_title_sort(self):
        """Returns a string used for sorting in the catalog."""
        sort = ""
        if self.state == 'open':
            sort += "0"
        elif self.state == 'suspended':
            sort += "1"
        elif self.state == 'closed':
            sort += "2"
        elif self.state == 'discarded':
            sort += "3"
        else:
            utilities.log("Unknown state %s" % self.state)
        # The dictionary hack is needed so that the catalog sorts
        # in the right order.  Should definently fix this in a
        # better way.
        sort += str({0:2,1:1,2:0}[self.importance])
        sort += self.get_title()
        return sort

    security.declarePrivate('get_open_goals')
    def get_open_goals(self):
        """Returns a list of open goals."""
        return self.catalog_search(path=self.absolute_url(relative=1),
                                   meta_type='Issue', type='goal', state='open')

    security.declarePrivate('get_closed_goals')
    def get_closed_goals(self):
        """Returns a list of closed goals."""
        return self.catalog_search(path=self.absolute_url(relative=1),
                                   meta_type='Issue', type='goal', state='closed')

    security.declarePrivate('get_open_problems')
    def get_open_problems(self):
        """Returns a list of open problems."""
        return self.catalog_search(path=self.absolute_url(relative=1),
                                   meta_type='Issue', type='problem', state='open')

    security.declarePrivate('get_closed_problems')
    def get_closed_problems(self):
        """Returns a list of closed problems."""
        return self.catalog_search(path=self.absolute_url(relative=1),
                                   meta_type='Issue', type='problem', state='closed')

    security.declarePrivate('get_open_questions')
    def get_open_questions(self):
        """Returns a list of open problems."""
        return self.catalog_search(path=self.absolute_url(relative=1),
                                   meta_type='Issue', type='question', state='open')

    security.declarePrivate('get_closed_questions')
    def get_closed_questions(self):
        """Returns a list of closed problems."""
        return self.catalog_search(path=self.absolute_url(relative=1),
                                   meta_type='Issue', type='question', state='closed')

    security.declarePrivate('get_relations')
    def get_relations(self):
        """Returns a list contained relations."""
        if self.get_user_preferences().show_deleted:
            deleted = [0,1]
        else:
            deleted = 0
        return self.catalog_search(get_parent_url=self.absolute_url(relative=1),
                                   meta_type='Relation', deleted=deleted)

    security.declareProtected('Use Issue Dealer', 'get_short_url')
    def get_short_url(self):
        """Returns a short URL to the object."""
        return self.get_issue_dealer().absolute_url() + '/redirect_issue?issue=%s' % self.id

    security.declareProtected('Use Issue Dealer', 'open')
    def open(self, ids=[], REQUEST=None):
        """Opens the specified issues."""
        if not ids:
            issues = [self]
        else:
            issues = map(lambda x, self=self: self[x], ids)
        for issue in issues:
            issue.change_state('open')
        if REQUEST:
            REQUEST.RESPONSE.redirect(self.absolute_url())

    security.declareProtected('Use Issue Dealer', 'close')
    def close(self, ids=[], REQUEST=None):
        """Closes the specified issues."""
        if not ids:
            issues = [self]
        else:
            issues = map(lambda x, self=self: self[x], ids)
        for issue in issues:
            issue.change_state('closed')
        if REQUEST:
            REQUEST.RESPONSE.redirect(self.absolute_url())

    security.declareProtected('Use Issue Dealer', 'suspend')
    def suspend(self, ids=[], REQUEST=None):
        """Suspend the specified issues."""
        if not ids:
            issues = [self]
        else:
            issues = map(lambda x, self=self: self[x], ids)
        for issue in issues:
            issue.change_state('suspended')
        if REQUEST:
            REQUEST.RESPONSE.redirect(self.absolute_url())

    security.declareProtected('Use Issue Dealer', 'discard')
    def discard(self, ids=[], REQUEST=None):
        """Discards the specified issues."""
        if not ids:
            issues = [self]
        else:
            issues = map(lambda x, self=self: self[x], ids)
        for issue in issues:
            issue.change_state('discarded')
        if REQUEST:
            REQUEST.RESPONSE.redirect(self.absolute_url())

    # Untested
    security.declarePrivate('get_all_text')
    def get_all_text(self):
        """Returns all text on the object."""
        return self.title + " " + self.contents

InitializeClass(issue)
