from Globals import DTMLFile, MessageDialog, Persistent, InitializeClass
import OFS
import Acquisition
import AccessControl
from Products import ZCatalog
import base
import issue

def manage_add_relation(self, id=None, relation_='', title='',
                        referrer='', REQUEST=None):
    """Add an Relation."""
    if id is None:
        id = self.get_unique_id()
    relation_ = relation(id, relation_, title,
                   creator=self.get_user().get_id(),
                   owner=self.get_user().get_id())
    self._setObject(id, relation_)
    try:
        self.index_object()
    except AttributeError:
        pass
    if referrer and REQUEST is not None:
        REQUEST.RESPONSE.redirect(referrer)
    else:
        if REQUEST is not None:
            return self.manage_main(self, REQUEST)
        else:
            # To get the object in a context
            return self[id]

class relation(
    ZCatalog.CatalogAwareness.CatalogAware,
    OFS.Folder.Folder,
    Persistent,
    Acquisition.Implicit,
    AccessControl.Role.RoleManager,
    base.base,
    ): 
    """Relation class.

    An relation relates two issues, and may contain
    issues related to the relationship.
    """

    meta_type = 'Relation'
    deleted = 0

    # Needed for cutting and pasting objects.
    all_meta_types = [
        {'visibility': 'Global',
         'interfaces': [],
         'action': 'manage_add_issue',
         'permission': 'Add Issue',
         'name': 'Issue',
         'product': 'Issue Dealer',
         'instance': ''}
    ]

    manage_options = (OFS.Folder.Folder.manage_options[0],) + \
        (
            {'label': 'View',       'action': ''},
            {'label': 'Security',   'action': 'manage_access'},
        )

    security = AccessControl.ClassSecurityInfo()

    security.declareProtected('Use Issue Dealer',
      'index_html', 'manage_add_issue', 'relation_edit')
    index_html = DTMLFile('relation_index', globals())
    manage_add_relation = manage_add_relation
    manage_add_issue = issue.manage_add_issue
    edit = DTMLFile('relation_edit', globals())
    relation_edit = DTMLFile('relation_edit', globals())

    def __init__(self, id, relation, title='', creator='', owner=''): 
        """Initialise a new instance of Relation"""
        self.id = id
        self.relation = relation
        self.title = title
        self.creator = creator or 'unknown'
        self.owner_ = owner or 'unknown'

    security.declarePrivate('get_related_object')
    def get_related_object(self):
        """Returns the related object."""
        return self.get_object(self.relation)

    # Untested
    security.declareProtected('Use Issue Dealer', 'manage_edit')
    def manage_edit(self, title):
        """Updates the object with new values."""
        self.title = title
        self.index_object()
        self.REQUEST.RESPONSE.redirect(self.absolute_url())

InitializeClass(issue)
